@@ -118,7 +118,7 @@ static int
 alpha_register_reggroup_p (struct gdbarch *gdbarch, int regnum,
 			   const struct reggroup *group)
 {
-  /* Filter out any registers eliminated, but whose regnum is 
+  /* Filter out any registers eliminated, but whose regnum is
      reserved for backward compatibility, e.g. the vfp.  */
   if (*gdbarch_register_name (gdbarch, regnum) == '\0')
     return 0;
@@ -510,7 +510,7 @@ alpha_extract_return_value (struct type *valtype, struct regcache *regcache,
     }
 }
 
-/* Insert the given value into REGCACHE as if it was being 
+/* Insert the given value into REGCACHE as if it was being
    returned by a function.  */
 
 static void
@@ -637,7 +637,7 @@ alpha_after_prologue (CORE_ADDR pc)
   if (!find_pc_partial_function (pc, NULL, &func_addr, &func_end))
     return 0;
 
-  sal = find_pc_line (func_addr, 0);
+  sal = find_sal_for_pc (func_addr, 0);
   if (sal.end < func_end)
     return sal.end;
 
@@ -736,7 +736,7 @@ static const int stq_c_opcode = 0x2f;
 
 /* Checks for an atomic sequence of instructions beginning with a LDL_L/LDQ_L
    instruction and ending with a STL_C/STQ_C instruction.  If such a sequence
-   is found, attempt to step through it.  A breakpoint is placed at the end of 
+   is found, attempt to step through it.  A breakpoint is placed at the end of
    the sequence.  */
 
 static std::vector<CORE_ADDR>
@@ -748,7 +748,7 @@ alpha_deal_with_atomic_sequence (struct gdbarch *gdbarch, CORE_ADDR pc)
   unsigned int insn = alpha_read_insn (gdbarch, loc);
   int insn_count;
   int index;
-  int last_breakpoint = 0; /* Defaults to 0 (no breakpoints placed).  */  
+  int last_breakpoint = 0; /* Defaults to 0 (no breakpoints placed).  */
   const int atomic_sequence_length = 16; /* Instruction sequence length.  */
   int bc_insn_count = 0; /* Conditional branch instruction count.  */
 
@@ -757,7 +757,7 @@ alpha_deal_with_atomic_sequence (struct gdbarch *gdbarch, CORE_ADDR pc)
       && INSN_OPCODE (insn) != ldq_l_opcode)
     return {};
 
-  /* Assume that no atomic sequence is longer than "atomic_sequence_length" 
+  /* Assume that no atomic sequence is longer than "atomic_sequence_length"
      instructions.  */
   for (insn_count = 0; insn_count < atomic_sequence_length; ++insn_count)
     {
@@ -765,7 +765,7 @@ alpha_deal_with_atomic_sequence (struct gdbarch *gdbarch, CORE_ADDR pc)
       insn = alpha_read_insn (gdbarch, loc);
 
       /* Assume that there is at most one branch in the atomic
-	 sequence.  If a branch is found, put a breakpoint in 
+	 sequence.  If a branch is found, put a breakpoint in
 	 its destination address.  */
       if (INSN_OPCODE (insn) >= br_opcode)
 	{
@@ -800,7 +800,7 @@ alpha_deal_with_atomic_sequence (struct gdbarch *gdbarch, CORE_ADDR pc)
   breaks[0] = loc;
 
   /* Check for duplicated breakpoints.  Check also for a breakpoint
-     placed (branch instruction's destination) anywhere in sequence.  */ 
+     placed (branch instruction's destination) anywhere in sequence.  */
   if (last_breakpoint
       && (breaks[1] == breaks[0]
 	  || (breaks[1] >= pc && breaks[1] <= closing_insn)))
@@ -876,7 +876,7 @@ alpha_sigtramp_frame_unwind_cache (const frame_info_ptr &this_frame,
 static CORE_ADDR
 alpha_sigtramp_register_address (struct gdbarch *gdbarch,
 				 CORE_ADDR sigcontext_addr, int regnum)
-{ 
+{
   alpha_gdbarch_tdep *tdep = gdbarch_tdep<alpha_gdbarch_tdep> (gdbarch);
 
   if (regnum >= 0 && regnum < 32)
@@ -884,7 +884,7 @@ alpha_sigtramp_register_address (struct gdbarch *gdbarch,
   else if (regnum >= ALPHA_FP0_REGNUM && regnum < ALPHA_FP0_REGNUM + 32)
     return sigcontext_addr + tdep->sc_fpregs_offset + regnum * 8;
   else if (regnum == ALPHA_PC_REGNUM)
-    return sigcontext_addr + tdep->sc_pc_offset; 
+    return sigcontext_addr + tdep->sc_pc_offset;
 
   return 0;
 }
@@ -1093,7 +1093,7 @@ Otherwise, you told GDB there was a function where there isn't one, or\n\
    something about the traditional layout of alpha stack frames.  */
 
 struct alpha_heuristic_unwind_cache
-{ 
+{
   CORE_ADDR vfp;
   CORE_ADDR start_pc;
   trad_frame_saved_reg *saved_regs;
@@ -1123,7 +1123,7 @@ alpha_heuristic_analyze_probing_loop (struct gdbarch *gdbarch, CORE_ADDR *pc,
 	subq    REG_INDEX,0x1,REG_INDEX
 	lda     REG_PROBE,<immediate>(REG_PROBE)
 	bne     REG_INDEX, LOOP_START
- 
+
 	lda     sp,<immediate>(REG_PROBE)
 
      If anything different is found, the function returns without
@@ -1149,14 +1149,14 @@ alpha_heuristic_analyze_probing_loop (struct gdbarch *gdbarch, CORE_ADDR *pc,
   cur_frame_size -= MEM_DISP (insn);
 
   /* stq     zero,<immediate>(REG_PROBE) */
-  
+
   cur_pc += ALPHA_INSN_SIZE;
   insn = alpha_read_insn (gdbarch, cur_pc);
   if (INSN_OPCODE (insn) != stq_opcode
       || MEM_RA (insn) != 0x1f
       || MEM_RB (insn) != reg_probe)
     return;
-  
+
   /* subq    REG_INDEX,0x1,REG_INDEX */
 
   cur_pc += ALPHA_INSN_SIZE;
@@ -1168,9 +1168,9 @@ alpha_heuristic_analyze_probing_loop (struct gdbarch *gdbarch, CORE_ADDR *pc,
       || OPR_RA (insn) != reg_index
       || OPR_RC (insn) != reg_index)
     return;
-  
+
   /* lda     REG_PROBE,<immediate>(REG_PROBE) */
-  
+
   cur_pc += ALPHA_INSN_SIZE;
   insn = alpha_read_insn (gdbarch, cur_pc);
   if (INSN_OPCODE (insn) != lda_opcode
@@ -1403,7 +1403,7 @@ alpha_heuristic_frame_prev_register (const frame_info_ptr &this_frame,
      the correct place.  */
   if (regnum == ALPHA_PC_REGNUM)
     regnum = info->return_reg;
-  
+
   return trad_frame_get_prev_register (this_frame, info->saved_regs, regnum);
 }
 
@@ -1522,7 +1522,7 @@ alpha_fill_fp_regs (const struct regcache *regcache,
 
 /* Return nonzero if the G_floating register value in REG is equal to
    zero for FP control instructions.  */
-   
+
 static int
 fp_register_zero_p (LONGEST reg)
 {
@@ -1597,7 +1597,7 @@ alpha_next_pc (struct regcache *regcache, CORE_ADDR pc)
 	  case 0x35:              /* FBNE */
 	    regno += gdbarch_fp0_regnum (gdbarch);
 	}
-      
+
       rav = regcache_raw_get_signed (regcache, regno);
 
       switch (op)
@@ -1636,7 +1636,7 @@ alpha_next_pc (struct regcache *regcache, CORE_ADDR pc)
 	  break;
 
 	/* Floating point branches.  */
-	
+
 	case 0x31:              /* FBEQ */
 	  if (fp_register_zero_p (rav))
 	    goto branch_taken;
