@@ -1,6 +1,6 @@
 /* Target-dependent code for UltraSPARC.
 
-   Copyright (C) 2003-2024 Free Software Foundation, Inc.
+   Copyright (C) 2003-2025 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -47,12 +47,12 @@
    code can handle both.  */
 
 /* The M7 processor supports an Application Data Integrity (ADI) feature
-   that detects invalid data accesses.  When software allocates memory and 
-   enables ADI on the allocated memory, it chooses a 4-bit version number, 
-   sets the version in the upper 4 bits of the 64-bit pointer to that data, 
-   and stores the 4-bit version in every cacheline of the object.  Hardware 
-   saves the latter in spare bits in the cache and memory hierarchy. On each 
-   load and store, the processor compares the upper 4 VA (virtual address) bits 
+   that detects invalid data accesses.  When software allocates memory and
+   enables ADI on the allocated memory, it chooses a 4-bit version number,
+   sets the version in the upper 4 bits of the 64-bit pointer to that data,
+   and stores the 4-bit version in every cacheline of the object.  Hardware
+   saves the latter in spare bits in the cache and memory hierarchy. On each
+   load and store, the processor compares the upper 4 VA (virtual address) bits
    to the cacheline's version. If there is a mismatch, the processor generates
    a version mismatch trap which can be either precise or disrupting.
    The trap is an error condition which the kernel delivers to the process
@@ -132,7 +132,7 @@ static std::forward_list<sparc64_adi_info> adi_proc_list;
 
 /* Get ADI info for process PID, creating one if it doesn't exist.  */
 
-static sparc64_adi_info * 
+static sparc64_adi_info *
 get_adi_info_proc (pid_t pid)
 {
   auto found = std::find_if (adi_proc_list.begin (), adi_proc_list.end (),
@@ -152,7 +152,7 @@ get_adi_info_proc (pid_t pid)
     }
 }
 
-static adi_stat_t 
+static adi_stat_t
 get_adi_info (pid_t pid)
 {
   sparc64_adi_info *proc;
@@ -176,7 +176,7 @@ sparc64_forget_process (pid_t pid)
     {
       if ((*it).pid == pid)
 	{
-	  if ((*it).stat.tag_fd > 0) 
+	  if ((*it).stat.tag_fd > 0)
 	    target_fileio_close ((*it).stat.tag_fd, &target_errno);
 	  adi_proc_list.erase_after (pit);
 	  break;
@@ -245,7 +245,7 @@ adi_normalize_address (CORE_ADDR addr)
   return addr;
 }
 
-/* Align a normalized address - a VA with bit 59 sign extended into 
+/* Align a normalized address - a VA with bit 59 sign extended into
    ADI bits.  */
 
 static CORE_ADDR
@@ -287,7 +287,7 @@ adi_tag_fd (void)
   char cl_name[MAX_PROC_NAME_SIZE];
   snprintf (cl_name, sizeof(cl_name), "/proc/%ld/adi/tags", (long) pid);
   fileio_error target_errno;
-  proc->stat.tag_fd = target_fileio_open (NULL, cl_name, O_RDWR|O_EXCL, 
+  proc->stat.tag_fd = target_fileio_open (NULL, cl_name, O_RDWR|O_EXCL,
 					  false, 0, &target_errno);
   return proc->stat.tag_fd;
 }
@@ -529,9 +529,7 @@ adi_assign_command (const char *args, int from_tty)
   do_assign (next_address, cnt, version);
 }
 
-void _initialize_sparc64_adi_tdep ();
-void
-_initialize_sparc64_adi_tdep ()
+INIT_GDB_FILE (sparc64_adi_tdep)
 {
   add_basic_prefix_cmd ("adi", class_support,
 			_("ADI version related commands."),
@@ -786,7 +784,7 @@ static const char * const sparc64_register_names[] =
 #define SPARC64_NUM_REGS ARRAY_SIZE (sparc64_register_names)
 
 /* We provide the aliases %d0..%d62 and %q0..%q60 for the floating
-   registers as "psuedo" registers.  */
+   registers as "pseudo" registers.  */
 
 static const char * const sparc64_pseudo_register_names[] =
 {
@@ -1050,7 +1048,7 @@ sparc64_skip_prologue (struct gdbarch *gdbarch, CORE_ADDR start_pc)
      using the debugging information.  */
   if (find_pc_partial_function (start_pc, NULL, &func_start, &func_end))
     {
-      sal = find_pc_line (func_start, 0);
+      sal = find_sal_for_pc (func_start, 0);
 
       if (sal.end < func_end
 	  && start_pc <= sal.end)
@@ -1135,16 +1133,16 @@ sparc64_frame_prev_register (const frame_info_ptr &this_frame, void **this_cache
   return frame_unwind_got_register (this_frame, regnum, regnum);
 }
 
-static const struct frame_unwind sparc64_frame_unwind =
-{
+static const struct frame_unwind_legacy sparc64_frame_unwind (
   "sparc64 prologue",
   NORMAL_FRAME,
+  FRAME_UNWIND_ARCH,
   default_frame_unwind_stop_reason,
   sparc64_frame_this_id,
   sparc64_frame_prev_register,
   NULL,
   default_frame_sniffer
-};
+);
 
 
 static CORE_ADDR
@@ -1884,7 +1882,6 @@ sparc64_supply_gregset (const struct sparc_gregmap *gregmap,
   enum bfd_endian byte_order = gdbarch_byte_order (gdbarch);
   int sparc32 = (gdbarch_ptr_bit (gdbarch) == 32);
   const gdb_byte *regs = (const gdb_byte *) gregs;
-  gdb_byte zero[8] = { 0 };
   int i;
 
   if (sparc32)
@@ -1947,7 +1944,7 @@ sparc64_supply_gregset (const struct sparc_gregmap *gregmap,
     }
 
   if (regnum == SPARC_G0_REGNUM || regnum == -1)
-    regcache->raw_supply (SPARC_G0_REGNUM, &zero);
+    regcache->raw_supply_zeroed (SPARC_G0_REGNUM);
 
   if ((regnum >= SPARC_G1_REGNUM && regnum <= SPARC_O7_REGNUM) || regnum == -1)
     {
